@@ -11,10 +11,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "Vector.h"
 #include "DenseMatrix.h"
 #include "DenseVector.h"
 
+namespace FortranLinalg{
+
 template <typename TPrecision>
 class LinalgIO{
 
@@ -240,4 +241,6 @@ class LinalgIO{
 
 };
 
+}
+
 #endif
